import Flutter
import IDWiseSDK
import UIKit

public class IdwiseFlutterPlugin: NSObject, FlutterPlugin {
  static var channel: FlutterMethodChannel? = nil

  var journeyID = ""
  public static func register(with registrar: FlutterPluginRegistrar) {
    let mchannel = FlutterMethodChannel(
      name: "idwise_flutter", binaryMessenger: registrar.messenger())
    self.channel = mchannel
    let instance = IdwiseFlutterPlugin()
    registrar.addMethodCallDelegate(instance, channel: mchannel)

  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "getPlatformVersion":
      result("iOS " + UIDevice.current.systemVersion)
    case "initialize":
      var clientKey: String = ""
      var sdkTheme: IDWiseSDKTheme = IDWiseSDKTheme.systemDefault
      if let parameteres = call.arguments as? [String: Any] {
        if let clientkey = parameteres["clientKey"] as? String {
          clientKey = clientkey
        }
        if let theme = parameteres["theme"] as? String {
          if theme == "LIGHT" {
            sdkTheme = IDWiseSDKTheme.light
          } else if theme == "DARK" {
            sdkTheme = IDWiseSDKTheme.dark
          } else {
            sdkTheme = IDWiseSDKTheme.systemDefault
          }
        }

      }
      IDWise.initialize(clientKey: clientKey, theme: sdkTheme) { error in
        result("got some error")
        print("onInitializeError")
        if let err = error {
          IdwiseFlutterPlugin.channel?.invokeMethod(
            "onInitializeError",
            arguments: ["errorCode": err.code, "message": err.message] as [String: Any])
        }
      }
    case "startJourney":
      // receiving arguments from Dart side and consuming here

      var referenceNo: String = ""  // optional parameter
      var locale: String = "en"
      var journeyDefinitionId = ""
      if let parameteres = call.arguments as? [String: Any] {
        if let refNo = parameteres["referenceNo"] as? String {
          referenceNo = refNo
        }
        if let loc = parameteres["locale"] as? String {
          locale = loc
        }
        if let journeyDefId = parameteres["journeyDefinitionId"] as? String {
          journeyDefinitionId = journeyDefId
        }
      }

      IDWise.startJourney(
        journeyDefinitionId: journeyDefinitionId, referenceNumber: referenceNo, locale: locale,
        journeyDelegate: self)

    case "startDynamicJourney":
      // receiving arguments from Dart side and consuming here

      var referenceNo: String = ""  // optional parameter
      var locale: String = "en"
      var journeyDefinitionId = ""
      if let parameteres = call.arguments as? [String: Any] {
        if let refNo = parameteres["referenceNo"] as? String {
          referenceNo = refNo
        }
        if let loc = parameteres["locale"] as? String {
          locale = loc
        }
        if let journeyDefId = parameteres["journeyDefinitionId"] as? String {
          journeyDefinitionId = journeyDefId
        }
      }
      IDWise.startDynamicJourney(
        journeyDefinitionId: journeyDefinitionId, referenceNumber: referenceNo, locale: locale,
        journeyDelegate: self, stepDelegate: self)
      result("successfully started journey")
    case "startStep":
      if let parameteres = call.arguments as? [String: Any] {
        if let stepId = parameteres["stepId"] as? String {
          IDWise.startStep(stepId: stepId)
        }
      }
    case "resumeDynamicJourney":
      var locale: String = "en"
      var journeyDefinitionId = ""
      var journeyId = ""
      if let parameteres = call.arguments as? [String: Any] {
        if let loc = parameteres["locale"] as? String {
          locale = loc
        }
        if let journeyDefId = parameteres["journeyDefinitionId"] as? String {
          journeyDefinitionId = journeyDefId
        }
        if let journeyID = parameteres["journeyId"] as? String {
          journeyId = journeyID
        }
      }

      IDWise.resumeDynamicJourney(
        journeyDefinitionId: journeyDefinitionId, journeyId: journeyId, locale: locale,
        journeyDelegate: self, stepDelegate: self)

    case "finishDynamicJourney":
      if let parameteres = call.arguments as? [String: Any] {
        if let journeyId = parameteres["journeyId"] as? String {
          IDWise.finishDynamicJourney(journeyId: journeyId)
        }
      }

    case "getJourneySummary":
      if let parameteres = call.arguments as? [String: Any] {
        if let journeyId = parameteres["journeyId"] as? String {
          IDWise.getJourneySummary(journeyId: journeyId) { summary, error in

            do {
              let jsonData = try JSONEncoder().encode(summary)
              var jsonString: Any?
              jsonString = try? JSONSerialization.jsonObject(
                with: jsonData, options: .mutableContainers)

              IdwiseFlutterPlugin.channel?.invokeMethod(
                "journeySummary",
                arguments: ["summary": jsonString, "error": error] as [String: Any])

            } catch {
              IdwiseFlutterPlugin.channel?.invokeMethod(
                "journeySummary",
                arguments: ["summary": "", "error": error] as [String: Any])

            }
          }
        }
      }

    case "unloadSDK":
      IDWise.unloadSDK()

    default:
      result(FlutterMethodNotImplemented)
    }
  }
}

extension IdwiseFlutterPlugin: IDWiseSDKJourneyDelegate {
  public func onJourneyResumed(journeyID: String) {
    self.journeyID = journeyID
    IdwiseFlutterPlugin.channel?.invokeMethod(
      "onJourneyResumed", arguments: ["journeyId": journeyID] as [String: Any])
  }

  public func onError(error: IDWiseSDKError) {
    IdwiseFlutterPlugin.channel?.invokeMethod(
      "onError", arguments: ["errorCode": error.code, "message": error.message] as [String: Any])
  }

  public func JourneyStarted(journeyID: String) {
    self.journeyID = journeyID
    IdwiseFlutterPlugin.channel?.invokeMethod(
      "onJourneyStarted", arguments: ["journeyId": journeyID] as [String: Any])
  }

  public func JourneyFinished() {
    IdwiseFlutterPlugin.channel?.invokeMethod(
      "onJourneyCompleted", arguments: ["journeyId": self.journeyID] as [String: Any])
  }

  public func JourneyCancelled() {
    IdwiseFlutterPlugin.channel?.invokeMethod(
      "onJourneyCancelled", arguments: ["journeyId": self.journeyID] as [String: Any])
  }

}

extension IdwiseFlutterPlugin: IDWiseSDKStepDelegate {
  public func onStepCaptured(stepId: Int, capturedImage: UIImage?) {

    let base64EncodedString = convertImageToBase64String(img: capturedImage)
    let response =
      ["stepId": "\(stepId)", "capturedImage": base64EncodedString] as [String: String]

    //if let jsonData = try? JSONEncoder().encode(response) {
    IdwiseFlutterPlugin.channel?.invokeMethod(
      "onStepCaptured",
      arguments: response)
    //}

  }

  public func onStepResult(stepId: Int, stepResult: IDWiseSDK.StepResult?) {

    do {
      let jsonData = try JSONEncoder().encode(stepResult)
      var jsonString: Any?
      jsonString = try? JSONSerialization.jsonObject(with: jsonData, options: .mutableContainers)

      IdwiseFlutterPlugin.channel?.invokeMethod(
        "onStepResult",
        arguments: ["stepId": stepId, "stepResult": jsonString] as [String: Any])

    } catch {
      IdwiseFlutterPlugin.channel?.invokeMethod(
        "onStepResult",
        arguments: ["stepId": stepId] as [String: Any])

    }

  }

  public func onStepConfirmed(stepId: String) {

  }

  func convertImageToBase64String(img: UIImage?) -> String {
    return img?.jpegData(compressionQuality: 1)?.base64EncodedString() ?? ""
  }

}
